@@ -12,7 +12,7 @@ from .param import (
     PARAM_MAX_SOURCE_ID,
     PARAM_MAX_VOLUME,
     PARAM_MAX_VOLUME_ZONEX,
-    PARAM_VOLUME_BOUNCE_WORKAROUND,
+    PARAM_POWER_ON_VOLUME_BOUNCE,
     PARAM_VOLUME_STEP_ONLY,
     PARAM_VOLUME_STEP_DELTA,
     PARAM_DEBUG_LISTENER,
@@ -25,6 +25,8 @@ from .param import (
 
 _LOGGER = logging.getLogger(__name__)
 
+VERSION = "0.1.4"
+
 PIONEER_COMMANDS = {
     "turn_on": {
         "1": ["PO", "PWR"],
@@ -115,7 +117,8 @@ class PioneerAVR:
         scan_interval=60,
         params=None,
     ):
-        """Initialize the Pioneer AVR interface."""
+        """Initialise the Pioneer AVR interface."""
+        _LOGGER.info("Starting aiopioneer %s", VERSION)
         _LOGGER.debug(
             '>> PioneerAVR.__init__(host="%s", port=%s, timeout=%s, params=%s)',
             host,
@@ -151,6 +154,7 @@ class PioneerAVR:
         self._disconnect_lock = asyncio.Lock()
         self._update_lock = asyncio.Lock()
         self._request_lock = asyncio.Lock()
+        self._update_event = asyncio.Event()
         self._reconnect = True
         self._full_update = True
         self._last_updated = 0.0
@@ -171,6 +175,10 @@ class PioneerAVR:
     def __del__(self):
         _LOGGER.debug(">> PioneerAVR.__del__()")
 
+    def get_unique_id(self):
+        """Get unique identifier for this instance."""
+        return self._host + ":" + str(self._port)
+
     ## Parameter management functions
     def _set_params(self):
         """Set current parameters."""
@@ -180,6 +188,7 @@ class PioneerAVR:
 
     def set_user_params(self, params=None):
         """Set parameters and merge with defaults."""
+        _LOGGER.debug(">> PioneerAVR.set_user_params(%s)", params)
         self._user_params = {}
         if params is not None:
             self._user_params = params
@@ -193,7 +202,7 @@ class PioneerAVR:
             for model_regex, model_params in PARAM_MODEL_DEFAULTS.items():
                 if re.search(model_regex, model):
                     _LOGGER.info(
-                        "Applying default parameters for model %s (%s)",
+                        "applying default parameters for model %s (%s)",
                         model,
                         model_regex,
                     )
@@ -201,23 +210,27 @@ class PioneerAVR:
         self._set_params()
 
     def get_params(self):
-        """Get all current parameters."""
-        return self._params
+        """Get a copy of all current parameters."""
+        params = {}
+        merge(params, self._params)
+        return params
 
     def get_user_params(self):
-        """Get user parameters."""
-        return self._user_params
+        """Get a copy of user parameters."""
+        params = {}
+        merge(params, self._user_params)
+        return params
 
     def get_default_params(self):
-        """Get default parameters."""
-        return self._default_params
+        """Get a copy of current default parameters."""
+        params = {}
+        merge(params, self._default_params)
+        return params
 
     ## Connection/disconnection
-    async def connect(self, from_reconnect=False):
+    async def connect(self, reconnect=True):
         """Open connection to AVR and start listener thread."""
-        _LOGGER.debug(
-            ">> PioneerAVR.connect(from_reconnect=%s) started", from_reconnect
-        )
+        _LOGGER.debug(">> PioneerAVR.connect() started")
         if self._connect_lock.locked():
             _LOGGER.warning("AVR connection is already connecting, skipping connect")
             return
@@ -226,7 +239,7 @@ class PioneerAVR:
             return
 
         async with self._connect_lock:
-            _LOGGER.debug("Opening AVR connection")
+            _LOGGER.debug("opening AVR connection")
             if self._writer is not None:
                 raise RuntimeError("AVR connection already established")
 
@@ -238,6 +251,7 @@ class PioneerAVR:
             self._reader = reader
             self._writer = writer
             self.available = True
+            self._reconnect = reconnect
             self._set_socket_options()
 
             await self._responder_cancel()
@@ -264,6 +278,7 @@ class PioneerAVR:
 
     async def set_scan_interval(self, scan_interval):
         """Set scan interval and restart updater."""
+        await self._updater_cancel()
         self.scan_interval = scan_interval
         await self._updater_schedule()
 
@@ -281,7 +296,7 @@ class PioneerAVR:
             return
 
         async with self._disconnect_lock:
-            _LOGGER.debug("Disconnecting AVR connection")
+            _LOGGER.debug("disconnecting AVR connection")
             self.available = False
             self._call_zone_callbacks()
 
@@ -293,7 +308,7 @@ class PioneerAVR:
             writer = self._writer
             if writer:
                 ## Close AVR connection
-                _LOGGER.debug("Closing AVR connection")
+                _LOGGER.debug("closing AVR connection")
                 self._writer.close()
                 try:
                     await self._writer.wait_closed()
@@ -319,24 +334,26 @@ class PioneerAVR:
         _LOGGER.debug(">> PioneerAVR.reconnect() started")
         retry = 0
         try:
-            while True:
+            while not self.available:
                 delay = get_backoff_delay(retry)
-                _LOGGER.debug("Waiting %ds before retrying connection", delay)
+                _LOGGER.debug("waiting %ds before retrying connection", delay)
                 await asyncio.sleep(delay)
 
                 retry += 1
                 try:
-                    await self.connect(from_reconnect=True)
-                    _LOGGER.debug("Scheduling full AVR status update")
-                    self._full_update = True
-                    await self.update()
-                    break
+                    await self.connect()
+                    ## 20201212 removed as connect already schedules full update
+                    # _LOGGER.debug("Scheduling full AVR status update")
+                    # self._full_update = True
+                    # await self.update()
+                    if self.available:
+                        break
                 except asyncio.CancelledError as exc:  # pylint: disable=try-except-raise
                     ## pass through to outer except
                     raise
                 except Exception as exc:  # pylint: disable=broad-except
                     _LOGGER.debug(
-                        "Could not reconnect to AVR: %s: %s", type(exc).__name__, exc
+                        "could not reconnect to AVR: %s: %s", type(exc).__name__, exc
                     )
                     ## fall through to reconnect outside try block
 
@@ -353,10 +370,11 @@ class PioneerAVR:
             _LOGGER.debug(">> PioneerAVR._reconnect_schedule()")
             reconnect_task = self._reconnect_task
             if reconnect_task:
+                await asyncio.sleep(0)  ## yield to reconnect task if running
                 if reconnect_task.done():
                     reconnect_task = None  ## trigger new task creation
             if reconnect_task is None:
-                _LOGGER.info("Reconnecting to AVR")
+                _LOGGER.info("reconnecting to AVR")
                 reconnect_task = asyncio.create_task(self.reconnect())
                 self._reconnect_task = reconnect_task
             else:
@@ -370,8 +388,9 @@ class PioneerAVR:
     async def _connection_listener(self):
         """AVR connection listener. Parse responses and update state."""
         _LOGGER.debug(">> PioneerAVR._connection_listener() started")
-        try:
-            while self.available:
+        running = True
+        while self.available:
+            try:
                 response = await self._read_response()
                 if response is None:
                     ## Connection closed or exception, exit task
@@ -383,7 +402,7 @@ class PioneerAVR:
                 if not response:
                     ## Skip processing empty responses (keepalives)
                     if debug_listener:
-                        _LOGGER.debug("Ignoring empty response")
+                        _LOGGER.debug("ignoring empty response")
                     continue
                 if debug_listener:
                     _LOGGER.debug("AVR listener received response: %s", response)
@@ -392,11 +411,11 @@ class PioneerAVR:
                 updated_zones = self._parse_response(response)
 
                 ## Detect Main Zone power on for volume workaround
-                volume_bounce_workaround = self._params[PARAM_VOLUME_BOUNCE_WORKAROUND]
-                if volume_bounce_workaround and self._power_zone_1 is not None:
+                power_on_volume_bounce = self._params[PARAM_POWER_ON_VOLUME_BOUNCE]
+                if power_on_volume_bounce and self._power_zone_1 is not None:
                     if not self._power_zone_1 and self.power["1"]:
                         ## Main zone powered on, schedule bounce task
-                        _LOGGER.info("Scheduling main zone volume workaround")
+                        _LOGGER.info("scheduling main zone volume workaround")
                         await self._bouncer_schedule()
                 self._power_zone_1 = self.power.get("1")  ## cache value
 
@@ -410,12 +429,19 @@ class PioneerAVR:
                     ## NOTE: updating zone 1 does not reset its scan interval -
                     ##       scan interval is set to a regular timer
 
-            if self.available:
-                ## Trigger disconnection if not already disconnected
-                await self.disconnect()
+            except asyncio.CancelledError:
+                _LOGGER.debug(">> PioneerAVR._connection_listener() cancelled")
+                running = False
+                break
+            except Exception as exc:  # pylint: disable=broad-except
+                _LOGGER.error(
+                    ">> PioneerAVR._connection_listener() exception: %s", str(exc)
+                )
+                # continue listening on exception
 
-        except asyncio.CancelledError:
-            _LOGGER.debug(">> PioneerAVR._connection_listener() cancelled")
+        if running and self.available:
+            ## Trigger disconnection if not already disconnected
+            await self.disconnect()
 
         _LOGGER.debug(">> PioneerAVR._connection_listener() completed")
 
@@ -457,11 +483,11 @@ class PioneerAVR:
             # responder_task = asyncio.create_task(self._reader.readuntil(b"\n"))
             self._responder_task = responder_task
             if debug_responder:
-                _LOGGER.debug("Created responder task %s", responder_task)
+                _LOGGER.debug("created responder task %s", responder_task)
         else:
             ## Wait on existing responder task
             if debug_responder:
-                _LOGGER.debug("Using existing responder task %s", responder_task)
+                _LOGGER.debug("using existing responder task %s", responder_task)
 
         ## Wait for result and process
         task_name = asyncio.current_task().get_name()
@@ -488,7 +514,7 @@ class PioneerAVR:
             _LOGGER.debug("%s: connection closed", task_name)
             return None
         except Exception as exc:  # pylint: disable=broad-except
-            _LOGGER.debug("%s: exception: %s", task_name, str(exc))
+            _LOGGER.error("%s: exception: %s", task_name, str(exc))
             return None
         if raw_response is None:  ## task cancelled
             return None
@@ -521,9 +547,9 @@ class PioneerAVR:
             command_delay = self._params[PARAM_COMMAND_DELAY]
             if since_command < command_delay:
                 delay = command_delay - since_command
-                _LOGGER.debug("Delaying command for %.3f s", delay)
+                _LOGGER.debug("delaying command for %.3f s", delay)
                 await asyncio.sleep(command_delay - since_command)
-        _LOGGER.debug("Sending AVR command: %s", command)
+        _LOGGER.debug("sending AVR command: %s", command)
         self._writer.write(command.encode("ASCII") + b"\r")
         await self._writer.drain()
         self._last_command = time.time()
@@ -598,45 +624,54 @@ class PioneerAVR:
                         _LOGGER.debug("send_command received response: %s", response)
                     return response
                 else:
-                    _LOGGER.error("Invalid request %s for zone %s", raw_command, zone)
+                    _LOGGER.error("invalid request %s for zone %s", raw_command, zone)
                     return None
             elif type(raw_command) is str:
                 return await self.send_raw_command(prefix + raw_command, rate_limit)
             else:
-                _LOGGER.warning("Invalid command %s for zone %s", command, zone)
+                _LOGGER.warning("invalid command %s for zone %s", command, zone)
                 return None
-        except RuntimeError:
-            _LOGGER.error("Cannot execute %s command: AVR not connected", command)
+        except RuntimeError as exc:
+            _LOGGER.error("cannot execute %s command: %s", command, exc)
             return False
 
     ## Initialisation functions
     async def query_zones(self):
         """Query zones on Pioneer AVR by querying power status."""
         if not self.zones:
-            _LOGGER.info("Querying available zones on AVR")
+            _LOGGER.info("querying available zones on AVR")
             ignored_zones = self._params[PARAM_IGNORED_ZONES]
-            if await self.send_command("query_power", "1", ignore_error=True):
+            if await self.send_command(
+                "query_power", "1", ignore_error=True
+            ) and await self.send_command("query_volume", "1", ignore_error=True):
                 if "1" not in self.zones and "1" not in ignored_zones:
                     _LOGGER.info("Zone 1 discovered")
                     self.zones.append("1")
                     self.max_volume["1"] = self._params[PARAM_MAX_VOLUME]
             else:
                 raise RuntimeError("Main Zone not found on AVR")
-            if await self.send_command("query_power", "2", ignore_error=True):
+            if await self.send_command(
+                "query_power", "2", ignore_error=True
+            ) and await self.send_command("query_volume", "2", ignore_error=True):
                 if "2" not in self.zones and "2" not in ignored_zones:
                     _LOGGER.info("Zone 2 discovered")
                     self.zones.append("2")
                     self.max_volume["2"] = self._params[PARAM_MAX_VOLUME_ZONEX]
-            if await self.send_command("query_power", "3", ignore_error=True):
+            if await self.send_command(
+                "query_power", "3", ignore_error=True
+            ) and await self.send_command("query_volume", "3", ignore_error=True):
                 if "3" not in self.zones and "3" not in ignored_zones:
                     _LOGGER.info("Zone 3 discovered")
                     self.zones.append("3")
                     self.max_volume["3"] = self._params[PARAM_MAX_VOLUME_ZONEX]
-            if await self.send_command("query_power", "Z", ignore_error=True):
+            if await self.send_command(
+                "query_power", "Z", ignore_error=True
+            ) and await self.send_command("query_volume", "Z", ignore_error=True):
                 if "Z" not in self.zones and "Z" not in ignored_zones:
                     _LOGGER.info("HDZone discovered")
                     self.zones.append("Z")
                     self.max_volume["Z"] = self._params[PARAM_MAX_VOLUME_ZONEX]
+            await self.update(full=True)
 
     def set_source_dict(self, sources):
         """Manually set source id<->name translation tables."""
@@ -646,29 +681,28 @@ class PioneerAVR:
     async def build_source_dict(self):
         """Generate source id<->name translation tables."""
         timeouts = 0
-        if not self._source_name_to_id:
-            _LOGGER.info("Querying AVR source names")
-            max_source_id = self._params[PARAM_MAX_SOURCE_ID]
-            for src_id in range(max_source_id):
-                response = await self.send_raw_request(
-                    "?RGB" + str(src_id).zfill(2),
-                    "RGB",
-                    ignore_error=True,
-                    rate_limit=False,
-                )
-                if response is None:
-                    timeouts += 1
-                    _LOGGER.debug("Timeout %d retrieving source %s", timeouts, src_id)
-                elif response is not False:
-                    timeouts = 0
-                    source_name = response[6:]
-                    source_active = response[5] == "1"
-                    source_number = str(src_id).zfill(2)
-                    if source_active:
-                        self._source_name_to_id[source_name] = source_number
-                        self._source_id_to_name[source_number] = source_name
-            _LOGGER.debug("Source name->id: %s", self._source_name_to_id)
-            _LOGGER.debug("Source id->name: %s", self._source_id_to_name)
+        self._source_name_to_id = {}
+        self._source_id_to_name = {}
+        _LOGGER.info("querying AVR source names")
+        max_source_id = self._params[PARAM_MAX_SOURCE_ID]
+        for src_id in range(max_source_id):
+            response = await self.send_raw_request(
+                "?RGB" + str(src_id).zfill(2),
+                "RGB",
+                ignore_error=True,
+                rate_limit=False,
+            )
+            if response is None:
+                timeouts += 1
+                _LOGGER.debug("timeout %d retrieving source %s", timeouts, src_id)
+            elif response is not False:
+                timeouts = 0
+                source_name = response[6:]
+                source_number = str(src_id).zfill(2)
+                self._source_name_to_id[source_name] = source_number
+                self._source_id_to_name[source_number] = source_name
+        _LOGGER.debug("source name->id: %s", self._source_name_to_id)
+        _LOGGER.debug("source id->name: %s", self._source_id_to_name)
         if not self._source_name_to_id:
             _LOGGER.warning("no input sources found on AVR")
 
@@ -676,12 +710,19 @@ class PioneerAVR:
         """Return list of available input sources."""
         return list(self._source_name_to_id.keys())
 
+    def get_source_name(self, source_id):
+        """Return name for given source ID."""
+        if not self._source_name_to_id:
+            return source_id
+        else:
+            return self._source_id_to_name.get(source_id, source_id)
+
     async def query_device_info(self):
         """Query device information from Pioneer AVR."""
         if self.model or self.mac_addr or self.software_version:
             return
 
-        _LOGGER.info("Querying device information from Pioneer AVR")
+        _LOGGER.info("querying device information from Pioneer AVR")
         model = None
         mac_addr = None
         software_version = None
@@ -715,10 +756,14 @@ class PioneerAVR:
         self.mac_addr = mac_addr if mac_addr else "unknown"
         self.software_version = software_version if software_version else "unknown"
 
-        # TODO: Query via HTML page if all info is not available from command
-        # http://avr/1000/system_information.asp
+        # It is possible to query via HTML page if all info is not available
+        # via API commands: http://avr/1000/system_information.asp
+        # However, this is not compliant with Home Assistant ADR-0004:
+        #
+        # https://github.com/home-assistant/architecture/blob/master/adr/0004-webscraping.md
+        #
         # VSX-930 will report model and software version, but not MAC address.
-        # Unknown how iControlAV5 determines this on a routed network.
+        # It is unknown how iControlAV5 determines this on a routed network.
 
     ## Callback functions
     def set_zone_callback(self, zone, callback):
@@ -737,7 +782,7 @@ class PioneerAVR:
             if zone in self.zones and zone in self._zone_callback:
                 callback = self._zone_callback[zone]
                 if callback:
-                    _LOGGER.debug("Calling callback for zone %s", zone)
+                    _LOGGER.debug("calling callback for zone %s", zone)
                     callback()
 
     ## update_callback deprecated, use zone callback
@@ -834,55 +879,57 @@ class PioneerAVR:
                 _LOGGER.info("HDZone: Mute: %s", value)
 
         elif response.startswith("FN"):
-            raw_id = response[2:]
-            value = self._source_id_to_name.get(raw_id, raw_id)
-            if self.source.get("1") != value:
-                self.source["1"] = value
+            id = response[2:]
+            if self.source.get("1") != id:
+                self.source["1"] = id
                 updated_zones.add("1")
-                _LOGGER.info("Zone 1: Source: %s", value)
+                _LOGGER.info("Zone 1: Source: %s (%s)", id, self.get_source_name(id))
         elif response.startswith("Z2F"):
-            raw_id = response[3:]
-            value = self._source_id_to_name.get(raw_id, raw_id)
-            if self.source.get("2") != value:
-                self.source["2"] = value
+            id = response[3:]
+            if self.source.get("2") != id:
+                self.source["2"] = id
                 updated_zones.add("2")
-                _LOGGER.info("Zone 2: Source: %s", value)
+                _LOGGER.info("Zone 2: Source: %s (%s)", id, self.get_source_name(id))
         elif response.startswith("Z3F"):
-            raw_id = response[3:]
-            value = self._source_id_to_name.get(raw_id, raw_id)
-            if self.source.get("3") != value:
-                value = self.source["3"]
+            id = response[3:]
+            if self.source.get("3") != id:
+                self.source["3"] = id
                 updated_zones.add("3")
-                _LOGGER.info("Zone 3: Source: %s", value)
+                _LOGGER.info("Zone 3: Source: %s (%s)", id, self.get_source_name(id))
         elif response.startswith("ZEA"):
-            raw_id = response[3:]
-            value = self._source_id_to_name.get(raw_id, raw_id)
-            if self.source.get("Z") != value:
-                self.source["Z"] = value
+            id = response[3:]
+            if self.source.get("Z") != id:
+                self.source["Z"] = id
                 updated_zones.add("Z")
-                _LOGGER.info("HDZone: Source: %s", value)
+                _LOGGER.info("HDZone: Source: %s (%s)", id, self.get_source_name(id))
         return updated_zones
 
     async def _updater(self):
         """Perform update every scan_interval."""
         _LOGGER.debug(">> PioneerAVR._updater() started")
+        event = self._update_event
         while True:
             try:
-                await asyncio.sleep(self.scan_interval)
-                await self.update()
+                await self._updater_update()
+                event.clear()
+                # await asyncio.sleep(self.scan_interval)
+                await asyncio.wait_for(event.wait(), timeout=self.scan_interval)
+            except asyncio.TimeoutError:  ## update timer expired
+                continue
             except asyncio.CancelledError:
                 _LOGGER.debug(">> PioneerAVR._updater() cancelled")
                 break
             except Exception as exc:  # pylint: disable=broad-except
-                _LOGGER.debug(">> PioneerAVR._updater() exception: %s", str(exc))
+                _LOGGER.error(">> PioneerAVR._updater() exception: %s", str(exc))
                 break
         _LOGGER.debug(">> PioneerAVR._updater() completed")
 
     async def _updater_schedule(self):
         """Schedule/reschedule the update task."""
-        _LOGGER.debug(">> PioneerAVR._updater_schedule()")
-        await self._updater_cancel()
         if self.scan_interval:
+            _LOGGER.debug(">> PioneerAVR._updater_schedule()")
+            await self._updater_cancel()
+            self._full_update = True  ## always perform full update on schedule
             self._updater_task = asyncio.create_task(self._updater())
 
     async def _updater_cancel(self):
@@ -904,8 +951,11 @@ class PioneerAVR:
             ## Timeout occurred, indicates AVR disconnected
             raise TimeoutError("Timeout waiting for data")
 
-    async def update(self):
+    async def _updater_update(self):
         """Update AVR cached status."""
+        debug_updater = self._params[PARAM_DEBUG_UPDATER]
+        if debug_updater:
+            _LOGGER.debug(">> PioneerAVR._updater_update() started")
         if self._update_lock.locked():
             _LOGGER.warning("AVR update already running, skipping")
             return False
@@ -919,9 +969,10 @@ class PioneerAVR:
             now = time.time()
             since_updated = now - self._last_updated
             full_update = self._full_update
-            if full_update or since_updated > self.scan_interval:
+            scan_interval = self.scan_interval
+            if full_update or not scan_interval or since_updated > scan_interval:
                 _LOGGER.info(
-                    "Updating AVR status (full=%s, last updated %.3f s ago)",
+                    "updating AVR status (full=%s, last updated %.3f s ago)",
                     full_update,
                     since_updated,
                 )
@@ -935,7 +986,7 @@ class PioneerAVR:
                         self._call_zone_callbacks()
                 except Exception as exc:  # pylint: disable=broad-except
                     _LOGGER.error(
-                        "Could not update AVR status: %s: %s",
+                        "could not update AVR status: %s: %s",
                         type(exc).__name__,
                         str(exc),
                     )
@@ -950,16 +1001,28 @@ class PioneerAVR:
                 ## Keepalives may be sent by the AVR (every 30 seconds on the
                 ## VSX-930) when connected to port 8102, but are not sent when
                 ## connected to port 23.
-                debug_updater = self._params[PARAM_DEBUG_UPDATER]
+                _rc = None
                 if debug_updater:
                     _LOGGER.debug(
-                        "Skipping update: last updated %.3f s ago", since_updated
+                        "skipping update: last updated %.3f s ago", since_updated
                     )
         if _rc is False:
             ## Disconnect on error
             await self.disconnect()
+        if debug_updater:
+            _LOGGER.debug(">> PioneerAVR._updater_update() completed")
         return _rc
 
+    async def update(self, full=False):
+        """Update AVR cached status update. Schedule if updater is running."""
+        if full:
+            self._full_update = True
+        if self._updater_task:
+            self._update_event.set()
+        else:
+            ## scan_interval not set, execute update synchronously
+            await self._updater_update()
+
     ## State change functions
     def _check_zone(self, zone):
         """Check that specified zone is valid."""
@@ -985,7 +1048,7 @@ class PioneerAVR:
                 "select_source", zone, prefix=source_id, ignore_error=False
             )
         else:
-            _LOGGER.error("Invalid source %s for zone %s", source, zone)
+            _LOGGER.error("invalid source %s for zone %s", source, zone)
             return False
 
     async def volume_up(self, zone="1"):
